if select(3, UnitClass("player")) ~= 9 then return end -- Only load when playing a warlock
local Whipped = _G.Whipped
local tinsert, sformat = table.insert, string.format
local EffectiveScale = UIParent:GetEffectiveScale()
local COMBATLOG_OBJECT_AFFILIATION_MINE = COMBATLOG_OBJECT_AFFILIATION_MINE
local Demons = Whipped.Demons
local Darkglare = Demons.Darkglare

-- Frames
local Frame_Darkglare = CreateFrame("StatusBar", nil, Whipped.MasterFrame)
local Frame_Darkglare_Icon = CreateFrame("Frame", nil, Frame_Darkglare)

function Whipped:Darkglare_SetupOptions()
	local Darkglare = {
		name = 'Darkglare',
		type = 'group',
		args = {
			Thickness = {
				name = 'Thickness',
				order = 1,
				type = 'range',
				min = 2,
				max = 200,
				softMin = 2,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Darkglare.Thickness = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Darkglare.Thickness end
			},
			Padding = {
				name = 'Padding',
				order = 2,
				type = 'range',
				min = 0,
				max = 200,
				softMin = 0,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Darkglare.Padding = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Darkglare.Padding end
			}
		}
	}
	self:AddCategory("Darkglare", Darkglare)
end

function Whipped:Darkglare_OnLoad()
	Frame_Darkglare:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
	Frame_Darkglare:SetMinMaxValues(0, 12)
	Frame_Darkglare:SetStatusBarColor(.1,.9,.2)

	Frame_Darkglare_Icon:SetWidth(Whipped.db.global.Darkglare.Thickness * EffectiveScale)
	Frame_Darkglare_Icon:SetHeight(Whipped.db.global.Darkglare.Thickness * EffectiveScale)
	Frame_Darkglare_Icon:SetPoint(Whipped.anchors[Whipped.db.global.Anchor], Frame_Darkglare, Whipped:OppositeAnchor(Whipped.db.global.Anchor))
	
	Frame_Darkglare_Icon.texture = Frame_Darkglare_Icon:CreateTexture(nil, "BACKGROUND")
	Frame_Darkglare_Icon.texture:SetTexture("Interface\\Icons\\achievement_boss_durumu")
	Frame_Darkglare_Icon.texture:SetAllPoints(true)
end

function Whipped:Darkglare_OnUpdate()
	if Darkglare.Summoned then -- Are Summoned or on Cooldown
		local t = GetTime()
		if Darkglare.TTL > 0 then -- Living Darkglare
			Darkglare.TTL = Darkglare.CD - t - 12
			Frame_Darkglare:SetValue(Darkglare.TTL)
		else -- Dead Darkglare
			if Darkglare.TTL < 0 then -- Statement block only runs once every summon, Setup for cooldown
				Darkglare.TTL = 0
				Darkglare.Alive = 0
				--DEBUG
				-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-15s".."|".."%-20s".."| "..Demons.ActiveDemons(),
					-- "UNIT_DESPAWNED",
					-- "'Darkglare'",
					-- "'Summon Darkglare'"))
				------
				Frame_Darkglare:SetStatusBarColor(.1,.1,.9)
			end
			local val = (12 - (Darkglare.CD - t)) -- (CD(24) - TTL(12)) / TTL(12) = 1
			Frame_Darkglare:SetValue(val)
			if val > 12 then -- Statement block only runs once every cast, Check if off cooldown (reset the whole thing)
				Darkglare.Dead = 0
				Darkglare.GUID = 0
				Darkglare.Summoned = false
				Frame_Darkglare:SetStatusBarColor(.1,.9,.2)
			end
		end
	end
end

function Whipped:Darkglare_UpdateFrames()
	local anchorID = Whipped.db.global.Anchor
	local anchor, Oanchor = Whipped.anchors[anchorID], Whipped:OppositeAnchor(anchorID)
	local SoulShards = Whipped.db.global.SoulShards
	local Grimoire = Whipped.db.global.Grimoire
	local Darkglare = Whipped.db.global.Darkglare
	
	if anchorID == 1 or anchorID == 2 then
		Frame_Darkglare:SetWidth((SoulShards.Width * 5 + SoulShards.Padding * 4) * EffectiveScale)
		Frame_Darkglare:SetHeight(Darkglare.Thickness * EffectiveScale)
	else
		Frame_Darkglare:SetWidth(Darkglare.Thickness * EffectiveScale)
		Frame_Darkglare:SetHeight((SoulShards.Height * 5 + SoulShards.Padding * 4) * EffectiveScale)
	end
	Frame_Darkglare_Icon:SetWidth(Darkglare.Thickness * EffectiveScale)
	Frame_Darkglare_Icon:SetHeight(Darkglare.Thickness * EffectiveScale)
		
	Frame_Darkglare:ClearAllPoints()
	Frame_Darkglare_Icon:ClearAllPoints()
	
	local anchorFrame
	if Demons.GrimoireDemon.Talented then
		anchorFrame = Whipped.Frame_Grimoire
	else
		anchorFrame = Whipped.Frame_Cooldown
	end
	
	if anchorID == 1 then
		Frame_Darkglare:SetPoint("TOP"..anchor, anchorFrame, "BOTTOM"..anchor,
		0, -(Darkglare.Padding + Grimoire.Padding) * EffectiveScale)
		Frame_Darkglare:SetOrientation("HORIZONTAL")
	elseif anchorID == 2 then
		Frame_Darkglare:SetPoint(anchor, anchorFrame, anchor,
		0, -(Darkglare.Padding + Grimoire.Padding + SoulShards.Height) * EffectiveScale)
		Frame_Darkglare:SetOrientation("HORIZONTAL")
	elseif anchorID == 3 then
		Frame_Darkglare:SetPoint(anchor.."LEFT", anchorFrame, anchor.."RIGHT",
		(Darkglare.Padding + Grimoire.Padding) * EffectiveScale, 0)
		Frame_Darkglare:SetOrientation("VERTICAL")
	elseif anchorID ==4 then
		Frame_Darkglare:SetPoint(anchor.."LEFT", anchorFrame, anchor.."RIGHT",
		(Darkglare.Padding + Grimoire.Padding) * EffectiveScale, 0)
		Frame_Darkglare:SetOrientation("VERTICAL")
	end
	if anchorID == 1 then
		Frame_Darkglare_Icon:SetPoint(anchor, Frame_Darkglare, Oanchor, SoulShards.PaddingX * EffectiveScale, 0)
	elseif anchorID == 2 then
		Frame_Darkglare_Icon:SetPoint(anchor, Frame_Darkglare, Oanchor, -SoulShards.PaddingX * EffectiveScale, 0)
	elseif anchorID == 3 then
		Frame_Darkglare_Icon:SetPoint(anchor, Frame_Darkglare, Oanchor, 0, -SoulShards.PaddingY * EffectiveScale)
	elseif anchorID == 4 then
		Frame_Darkglare_Icon:SetPoint(anchor, Frame_Darkglare, Oanchor, 0, SoulShards.PaddingY * EffectiveScale)
	end
end

function Whipped:Darkglare_CLE(...)
	local event = select(2, ...)
	if event == "SPELL_CAST_SUCCESS" then
		if Darkglare.Alive > 0 then
			Frame_Darkglare:SetStatusBarColor(1,.1,.1)
		end
	elseif event == "SPELL_SUMMON" then
		local SpellID = select(12, ...)
		if SpellID == 205180 and bit.band(select(6,...), COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then -- Darkglare
			Darkglare.Summoned = true
			Darkglare.CD = GetTime() + 24
			Darkglare.TTL = Darkglare.CD - 12
			Darkglare.Alive = 1
			Darkglare.DE = 1
			Darkglare.GUID = select(8, ...)
			--DEBUG
			-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-15s".."|".."%-20s".."| "..Demons.ActiveDemons(),
				-- event,
				-- "'Darkglare'",
				-- "'Summon Darkglare'"))
			------
		end
	elseif event == "UNIT_DIED" then
		if select(8,...) == Demons.Darkglare.GUID then -- Darkglare
			Darkglare.Alive = 0
			Darkglare.TTL = 0
			Frame_Darkglare:SetStatusBarColor(.1,.1,.9)
			--DEBUG
			-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-15s".."|".."%-20s".."| "..Demons.ActiveDemons(),
				-- event,
				-- "'Darkglare'",
				-- "'Summon Darkglare'"))
			------
		end
	end
end

function Whipped:Darkglare_UpdateTalents()
	local id,_,_,b = GetTalentInfo(7,1,1)
	if id == 22478 and b then
		Darkglare.Talented = true
		Frame_Darkglare:Show()
	else
		Darkglare.Talented = false
		Frame_Darkglare:Hide()
	end
	--DEBUG
	-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-36s".."| "..tostring(Demons.Darkglare.Talented),
		-- "TALENT_UPDATE",
		-- "'Darkglare'"))
	------
end