if select(3, UnitClass("player")) ~= 9 then return end -- Only load when playing a warlock
local Whipped = _G.Whipped
local EffectiveScale = UIParent:GetEffectiveScale()

-- Frames
Whipped.Frame_SoulShard_1 = CreateFrame("StatusBar","SS1",Whipped.MasterFrame)
local Frame_SoulShard_1 = Whipped.Frame_SoulShard_1
local Frame_SoulShard_2 = CreateFrame("StatusBar","SS2",Frame_SoulShard_1)
local Frame_SoulShard_3 = CreateFrame("StatusBar","SS3",Frame_SoulShard_2)
local Frame_SoulShard_4 = CreateFrame("StatusBar","SS4",Frame_SoulShard_3)
local Frame_SoulShard_5 = CreateFrame("StatusBar","SS5",Frame_SoulShard_4)
local SoulShardsList = {Frame_SoulShard_1, Frame_SoulShard_2, Frame_SoulShard_3, Frame_SoulShard_4, Frame_SoulShard_5}

function Whipped:SoulShards_SetupOptions()
	local SoulShards = {
		name = 'SoulShards',
		type = 'group',
		args = {
			Width = {
				name = 'Width',
				order = 1,
				type = 'range',
				min = 1,
				max = 200,
				softMin = 2,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.SoulShards.Width = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.SoulShards.Width end
			},
			Height = {
				name = 'Height',
				order = 2,
				type = 'range',
				min = 2,
				max = 200,
				softMin = 2,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.SoulShards.Height = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.SoulShards.Height end
			},
			PaddingX = {
				name = 'Padding Horizontal',
				order = 4,
				type = 'range',
				min = 0,
				max = 200,
				softMin = 0,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.SoulShards.PaddingX = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.SoulShards.PaddingX end
			},
			PaddingY = {
				name = 'Padding Vertical',
				order = 5,
				type = 'range',
				min = 0,
				max = 200,
				softMin = 0,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.SoulShards.PaddingY = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.SoulShards.PaddingY end
			},
			Padding = {
				name = 'Padding',
				order = 3,
				type = 'range',
				min = 0,
				max = 200,
				softMin = 0,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.SoulShards.Padding = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.SoulShards.Padding end
			},
		}
	}
	self:AddCategory("SoulShards", SoulShards)
end

function Whipped:SoulShards_OnLoad()
	local SoulShards = Whipped.db.global.SoulShards
	for i=1,5 do
		SoulShardsList[i]:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
		SoulShardsList[i]:SetMinMaxValues(0, 1)
		SoulShardsList[i]:SetStatusBarColor(.90,0,1)
	end
end

function Whipped:SoulShards_OnUpdate()
	local up = UnitPower("player", 7)
	for i=1, up do
		if i < up+1 then
			SoulShardsList[i]:Show()
		else
			SoulShardsList[i]:Hide()
		end
	end
	for i=up+1, 5 do
		SoulShardsList[i]:Hide()
	end
end

function Whipped:SoulShards_UpdateFrames()
	local anchorID = Whipped.db.global.Anchor
	local anchor, Oanchor = Whipped.anchors[anchorID], Whipped:OppositeAnchor(anchorID)
	local SoulShards = Whipped.db.global.SoulShards
	
	for i=1,5 do
		SoulShardsList[i]:SetWidth(SoulShards.Width * EffectiveScale)
		SoulShardsList[i]:SetHeight(SoulShards.Height * EffectiveScale)
	end
	
	SoulShardsList[1]:ClearAllPoints()
	if anchorID == 1 then
		SoulShardsList[1]:SetPoint("TOP"..anchor, Whipped.MasterFrame, "TOP"..anchor, SoulShards.PaddingX * EffectiveScale, -SoulShards.PaddingY * EffectiveScale)
	elseif anchorID == 2 then
		SoulShardsList[1]:SetPoint("TOP"..anchor, Whipped.MasterFrame, "TOP"..anchor, -SoulShards.PaddingX * EffectiveScale, -SoulShards.PaddingY * EffectiveScale)
	elseif anchorID == 3 then
		SoulShardsList[1]:SetPoint(anchor.."LEFT", Whipped.MasterFrame, anchor.."LEFT", SoulShards.PaddingX * EffectiveScale, -SoulShards.PaddingY * EffectiveScale)
	elseif anchorID ==4 then
		SoulShardsList[1]:SetPoint(anchor.."LEFT", Whipped.MasterFrame, anchor.."LEFT", SoulShards.PaddingX * EffectiveScale, SoulShards.PaddingY * EffectiveScale)
	end
	for i=1,4 do
		SoulShardsList[i+1]:ClearAllPoints()
		if anchorID == 1 then
			SoulShardsList[i+1]:SetPoint("TOP"..anchor, "SS"..i, "TOP"..Oanchor, SoulShards.Padding * EffectiveScale, 0)
		elseif anchorID == 2 then
			SoulShardsList[i+1]:SetPoint("TOP"..anchor, "SS"..i, "TOP"..Oanchor, -SoulShards.Padding * EffectiveScale, 0)
		elseif anchorID == 3 then
			SoulShardsList[i+1]:SetPoint(anchor.."LEFT", "SS"..i, Oanchor.."LEFT", 0, -SoulShards.Padding * EffectiveScale)
		elseif anchorID ==4 then
			SoulShardsList[i+1]:SetPoint(anchor.."LEFT", "SS"..i, Oanchor.."LEFT", 0, SoulShards.Padding * EffectiveScale)
		end
	end
end